@@ -1,10 +1,9 @@
 import copy
-import logging
 from typing import TYPE_CHECKING
 
 import naff.api.events as events
 
-from naff.client.const import logger_name, MISSING
+from naff.client.const import MISSING
 from ._template import EventMixinTemplate, Processor
 
 if TYPE_CHECKING:
@@ -12,8 +11,6 @@ if TYPE_CHECKING:
 
 __all__ = ("MemberEvents",)
 
-log = logging.getLogger(logger_name)
-
 
 class MemberEvents(EventMixinTemplate):
     @Processor.define()
