@@ -1,6 +1,5 @@
 """This file handles the interaction with discords http endpoints."""
 import asyncio
-import logging
 from typing import Any, Optional
 from urllib.parse import quote as _uriquote
 from weakref import WeakValueDictionary
@@ -29,7 +28,7 @@ from naff.client.const import (
     __py_version__,
     __repo_url__,
     __version__,
-    logger_name,
+    logger,
     MISSING,
     Absent,
     __api_version__,
@@ -40,11 +39,10 @@ from naff.client.utils.serializer import dict_filter_missing
 from naff.models import CooldownSystem
 from naff.models.discord.file import UPLOADABLE_TYPE
 from .route import Route
+import discord_typings
 
 __all__ = ("HTTPClient",)
 
-log = logging.getLogger(logger_name)
-
 
 class GlobalLock:
     """Manages the global ratelimit"""
@@ -115,10 +113,10 @@ class BucketLock:
         loop = asyncio.get_running_loop()
         loop.call_later(self.delta, self.unlock)
 
-    async def defer_unlock(self) -> None:
+    async def defer_unlock(self, reset_after: float | None = None) -> None:
         """Unlocks the BucketLock after a specified delay."""
         self.unlock_on_exit = False
-        await asyncio.sleep(self.delta)
+        await asyncio.sleep(reset_after or self.delta)
         self.unlock()
 
     async def __aenter__(self) -> None:
@@ -194,7 +192,7 @@ class HTTPClient(
 
         if bucket_lock.bucket_hash:
             # We only ever try and cache the bucket if the bucket hash has been set (ignores unlimited endpoints)
-            log.debug(f"Caching ingested rate limit data for: {bucket_lock.bucket_hash}")
+            logger.debug(f"Caching ingested rate limit data for: {bucket_lock.bucket_hash}")
             self._endpoints[route.rl_bucket] = bucket_lock.bucket_hash
             self.ratelimit_locks[bucket_lock.bucket_hash] = bucket_lock
 
@@ -210,7 +208,7 @@ class HTTPClient(
         Returns:
             Either a dictionary or multipart data form
         """
-        if not payload:
+        if payload in (None, MISSING):
             return None
 
         if isinstance(payload, dict):
@@ -292,32 +290,42 @@ class HTTPClient(
 
                         if response.status == 429:
                             # ratelimit exceeded
-
                             if result.get("global", False):
+                                # global ratelimit is reached
                                 # if we get a global, that's pretty bad, this would usually happen if the user is hitting the api from 2 clients sharing a token
-                                log.error(
+                                logger.error(
                                     f"Bot has exceeded global ratelimit, locking REST API for {result.get('retry_after')} seconds"
                                 )
                                 await self.global_lock.lock(float(result.get("retry_after")))
                                 continue
+                            elif result.get("message") == "The resource is being rate limited.":
+                                # resource ratelimit is reached
+                                logger.warning(
+                                    f"{route.endpoint} The resource is being rate limited! "
+                                    f"Reset in {result.get('retry_after')} seconds"
+                                )
+                                # lock this resource and wait for unlock
+                                await lock.defer_unlock(float(result.get("retry_after")))
+                                continue
                             else:
+                                # endpoint ratelimit is reached
                                 # 429's are unfortunately unavoidable, but we can attempt to avoid them
                                 # so long as these are infrequent we're doing well
-                                log.warning(
+                                logger.warning(
                                     f"{route.endpoint} Has exceeded it's ratelimit ({lock.limit})! Reset in {lock.delta} seconds"
                                 )
                                 await lock.defer_unlock()  # lock this route and wait for unlock
                                 continue
                         elif lock.remaining == 0:
                             # Last call available in the bucket, lock until reset
-                            log.debug(
+                            logger.debug(
                                 f"{route.endpoint} Has exhausted its ratelimit ({lock.limit})! Locking route for {lock.delta} seconds"
                             )
                             await lock.blind_defer_unlock()  # lock this route, but continue processing the current response
 
                         elif response.status in {500, 502, 504}:
                             # Server issues, retry
-                            log.warning(
+                            logger.warning(
                                 f"{route.endpoint} Received {response.status}... retrying in {1 + attempt * 2} seconds"
                             )
                             await asyncio.sleep(1 + attempt * 2)
@@ -326,7 +334,7 @@ class HTTPClient(
                         if not 300 > response.status >= 200:
                             await self._raise_exception(response, route, result)
 
-                        log.debug(
+                        logger.debug(
                             f"{route.endpoint} Received {response.status} :: [{lock.remaining}/{lock.limit} calls remaining]"
                         )
                         return result
@@ -337,7 +345,7 @@ class HTTPClient(
                     raise
 
     async def _raise_exception(self, response, route, result) -> None:
-        log.error(f"{route.method}::{route.url}: {response.status}")
+        logger.error(f"{route.method}::{route.url}: {response.status}")
 
         if response.status == 403:
             raise Forbidden(response, response_data=result, route=route)
@@ -349,7 +357,7 @@ class HTTPClient(
             raise HTTPException(response, response_data=result, route=route)
 
     async def request_cdn(self, url, asset) -> bytes:
-        log.debug(f"{asset} requests {url} from CDN")
+        logger.debug(f"{asset} requests {url} from CDN")
         async with self.__session.get(url) as response:
             if response.status == 200:
                 return await response.read()
@@ -394,6 +402,13 @@ class HTTPClient(
             raise GatewayNotFound from exc
         return "{0}?encoding={1}&v={2}&compress=zlib-stream".format(data["url"], "json", __api_version__)
 
+    async def get_gateway_bot(self) -> discord_typings.GetGatewayBotData:
+        try:
+            data: dict = await self.request(Route("GET", "/gateway/bot"))
+        except HTTPException as exc:
+            raise GatewayNotFound from exc
+        return data
+
     async def websocket_connect(self, url: str) -> ClientWebSocketResponse:
         """
         Connect to the websocket.
